
// >> Background Script ========================================================

// debug modes
const kpmDebug_Errors = true;
const kpmDebug_LogInfo = false;

// kpmLog -----------------------------------------------------------------------

const kpmLog =
{
  logMessage: function(text)
  {
    try 
    {
      console.debug('Kaspersky KSOS: ' + text);
    }
    catch (ErrorMessage)
    {
      // keep silence
    }
  },
  
  logError: function(text)
  {
    if (kpmDebug_Errors)
    try
    {
      console.error('Kaspersky KSOS: ' + text);
    }
    catch (ErrorMessage)
    {
      // keep silence
    }
  }
}

// kpmAutofill ------------------------------------------------------------------

var kpmAutofill = null;
var spAuthKey = '';

function kpmGetAutofill()
{
  try
  {
    // try to get already added plugin
    kpmAutofill = document.getElementById('kpmAutofillPlugin');
    // initialize AuthKey from plugin
    if (kpmAutofill)
    {
      var AuthKey_ = {};
      if (kpmAutofill.GetAuthKey(AuthKey_))
        spAuthKey = AuthKey_.value;
    }
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmGetAutofill() Error: ' + ErrorMessage);
  }
}

// try to initialize kpmAutofill NPAPI
kpmGetAutofill();

// Tabs engine -----------------------------------------------------------------

function kpmCloseAllTabs()
{
  chrome.tabs.getAllInWindow(null,
    function (tabs)
    {
      try
      {
        for (var i = 0; i < tabs.length; i++)
        {
          var tab = tabs[i];
          chrome.tabs.remove(tab.id);
        }
      }
      catch (ErrorMessage)
      {
        kpmLog.logError('kpmCloseAllTabs() Error: ' + ErrorMessage);
      }
    }
  );
}

function kpmIsBlankTab(tab)
{
  var Result = false;
  if (tab)
  {
    if (kpmWideSameText(tab.url, 'about:blank') ||
        kpmWideSameText(tab.url, 'chrome://newtab/')
       )
    {
      Result = true;
    }
  }
  return Result;
}

function kpmOpenOrReuseTab(Url)
{
  // check each tab for our Url
  chrome.tabs.getAllInWindow(null,
    function (tabs)
    {
      try
      {
        var blankTab = null;
        for (var i = 0; i < tabs.length; i++)
        {
          var tab = tabs[i];
          if (kpmIsBlankTab(tab))
          {
            // tab is blank - store it
            blankTab = tab;
          } else
          if (kpmWideSameText(tab.url, Url) ||
              kpmWideSameText(tab.url, Url + '/')
             )
          {
            // the Url is already opened, select this tab & focus
            chrome.tabs.update(tab.id, {selected: true});
            return true;
          }
        }
        // navigate Url in blank tab or open Url in new tab
        if (blankTab)
        {
          chrome.tabs.update(blankTab.id, {selected: true, url: Url});
        }
        else
        {
          chrome.tabs.create({url: Url});
        }
      }
      catch (ErrorMessage)
      {
        kpmLog.logError('kpmOpenOrReuseTab() Error: ' + ErrorMessage);
      }
    }
  );
  return true;
}

function kpmIsAllowedTab(tab)
{
  var Result = false;
  if (tab)
  {
    if (kpmWideSearchText('http://', tab.url) ||
        kpmWideSearchText('https://', tab.url)
       )
    {
      Result = true;
    }
  }
  return Result;
}

chrome.tabs.onSelectionChanged.addListener(
  function(tabId, selectInfo)
  {
    try
    {
      if (kpmAutofill)
        kpmAutofill.HideShowedCaptionButtonBalloon();
      // post message to Content Script of all tabs
      chrome.tabs.getAllInWindow(selectInfo.windowId,
        function(tabs)
        {
          for (var i = 0; i < tabs.length; i++)
          {
            var tab = tabs[i];
            if (kpmIsAllowedTab(tab))
              chrome.tabs.sendRequest(tab.id, {name: 'kpmOnTabSelect', IsActiveTab: tab.id == tabId});
          }
        }
      );
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('chrome.tabs.onSelectionChanged() Error: ' + ErrorMessage);
    }
  }
);

// Timer engine ----------------------------------------------------------------

var kpmIsPasswordManagerUnlocked = -1;
var kpmInAutofillEngineOnTimer = false;
  
function kpmAutofillEngineOnTimer(AEvent)
{
  if (kpmInAutofillEngineOnTimer)
    return;
  try
  {
    chrome.tabs.getSelected(null,
      function(selectedTab)
      {
        if (!selectedTab)
          return;
        if (kpmInAutofillEngineOnTimer)
          return;
        kpmInAutofillEngineOnTimer = true;
        try
        {
          // post message to Content Script
          if (kpmIsAllowedTab(selectedTab) &&
              (kpmAutofill.HasWebBrowserTabMessage() ||      // we have any message in listener!
               kpmAutofill.AllowVirtualKeyboardAutoShow()    // is auto-show virtual keyboard allowed?
              )
             )
          {
            chrome.tabs.sendRequest(selectedTab.id, {name: 'kpmAutofillEngineOnTimerFocusedWindow'});
          }

          // try to process some actions of kpmAutofill
          if (kpmAutofill)
          {
            // access Listener from Background script
            kpmAutofill.AccessListener(false);
            
            // update tab icon
            var IsUnlocked = kpmAutofill.IsPasswordManagerUnlocked();
            if (kpmIsPasswordManagerUnlocked != IsUnlocked)
            {
              kpmIsPasswordManagerUnlocked = IsUnlocked;
              if (IsUnlocked)
                chrome.browserAction.setIcon({path: 'skin/logo16.png'});
              else
                chrome.browserAction.setIcon({path: 'skin/logo16-disabled.png'});
            }

            // try to set current Url
            try
            {
              var Url = selectedTab.url;
              kpmAutofill.SetUrl(Url);
            }
            catch (ErrorMessage)
            {
              kpmLog.logError('kpmAutofillEngineOnTimer() Error during SetUrl(): ' + ErrorMessage);
            }
            // check if we need to open new tab for specified Url
            var objMessageID = {};
            var objUrl = {};
            var objInActiveTab = {};
            if (kpmAutofill.MustExecuteUrl(selectedTab.title, objMessageID, objUrl, objInActiveTab))
            try
            {
              if (kpmDebug_LogInfo)
                kpmLog.logMessage('MustExecuteUrl(), Url: <' + objUrl.value + '> InActiveTab: ' + objInActiveTab.value);
              if (objInActiveTab.value)
                kpmCloseAllTabs();
              var Result = kpmOpenOrReuseTab(objUrl.value);            
              kpmAutofill.ExecuteUrl(objMessageID.value, Result);
              if (kpmDebug_LogInfo)
                kpmLog.logMessage('   ExecuteUrl() Result: ' + Result);
            }
            catch (ErrorMessage)
            {
              kpmAutofill.MessageProcessFailed(objMessageID.value);
              kpmLog.logError('kpmAutofillEngineOnTimer() Error during ExecuteUrl(): ' + ErrorMessage);
            }
          }
          kpmInAutofillEngineOnTimer = false;
        }
        catch (ErrorMessage)
        {
          kpmInAutofillEngineOnTimer = false;
          kpmLog.logError('kpmAutofillEngineOnTimer() Error: ' + ErrorMessage);
        }
        finally
        {
          kpmInAutofillEngineOnTimer = false;
        }
      }
    );    
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmAutofillEngineOnTimer() Error: ' + ErrorMessage);
  }
}

function kpmStartAutofillEngineTimer()
{
  try
  {
    setInterval('kpmAutofillEngineOnTimer()', 200);
  }
  catch (ErrorMessage)
  {
    kpmLog.logError('kpmStartAutofillEngineTimer() Error: ' + ErrorMessage);
  }
}

kpmStartAutofillEngineTimer();

chrome.browserAction.onClicked.addListener(
  function(tab)
  {
    try
    {
      if (kpmAutofill)
        kpmAutofill.ShowCaptionButtonMenu();
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('chrome.browserAction.onClicked() Error: ' + ErrorMessage);
    }
  }
);

chrome.extension.onRequest.addListener(
  function(request, sender, sendResponse)
  {
    // 1. process message    
    if (request)
    try
    {
      if (request.name == 'kpmGetAuthKey')
      {
        // this allows to return AuthKey to content script
        sendResponse({name: 'spAuthKey', AuthKey: spAuthKey});
        return;
      }
    }
    catch (ErrorMessage)
    {
      kpmLog.logError('chrome.extension.onRequest() Error: ' + ErrorMessage);
    }
    // 2. always return the response, otherwise we'll got the leaks!
    sendResponse({});
  }
);
  
// << Background Script ========================================================

// >> Strings Script ===========================================================

// log modes
var kpmLog_WideIsEmpty = false;
var kpmLog_WideSameText = false;
var kpmLog_WideSearchText = false;

const kpmLineBreak = '\r\n';
 
function kpmWidePrepareStr(Text)
{
  var Result;
  try
  {
    if (Text && Text.value != undefined)
      Result = new String(Text.value); // String object
    else
      Result = new String(Text);       // String
  }
  catch (ErrorMessage)
  {
    if (kpmDebug_Errors)
      kpmLog.logError('kpmWidePrepareStr() Error: ' + ErrorMessage);
    Result = new String(Text); // String
  }
  return Result;
}

function kpmWidePrepareLowerCaseStr(Text)
{
  var Result = kpmWidePrepareStr(Text);
  Result = Result.toLowerCase();
  return Result;  
}

function kpmWidePrepareUpperCaseStr(Text)
{
  var Result = kpmWidePrepareStr(Text);
  Result = Result.toUpperCase();
  return Result;  
}

function kpmWideIsEmpty(Text)
{
  try
  {
    if (typeof Text == 'object')
    {
      if (Text == null)
      {
        if (kpmLog_WideIsEmpty)
        {
          kpmLog.logMessage('kpmWideIsEmpty() Text[object]: <' + Text + '/' + Text.value + '>');
          kpmLog.logMessage('kpmWideIsEmpty() Text == null');
        }
        return true;
      }
      if ((Text.value == '') || (Text.value == undefined) || (Text.empty && Text.empty()))
      {
        if (kpmLog_WideIsEmpty)
        {
          kpmLog.logMessage('kpmWideIsEmpty() Text[object]: <' + Text + '/' + Text.value + '>');
          kpmLog.logMessage('kpmWideIsEmpty() Text.empty()');
        }
        return true;
      }        
    } else
    if (typeof Text == 'string')
    {      
      if (Text == '' || Text == null || Text == undefined)
      {
        if (kpmLog_WideIsEmpty)
        {
          kpmLog.logMessage('kpmWideIsEmpty() Text[string]: <' + Text + '>');
          kpmLog.logMessage('kpmWideIsEmpty() Text == "" || Text == null || Text == undefined');
        }
        return true;
      }
    } else
    { 
      // unknown type
      if (kpmLog_WideIsEmpty)
        kpmLog.logMessage('kpmWideIsEmpty() Text[' + typeof Text + ']: <' + Text + '>');
      return true;
    }
    return false;
  }
  catch (ErrorMessage)
  {
    if (kpmDebug_Errors)
      kpmLog.logError('kpmWideIsEmpty() Error: ' + ErrorMessage);
  }
  return false;  
}

function kpmWideSameText(Text1, Text2)
{
  try
  {
    var Str1 = kpmWidePrepareLowerCaseStr(Text1);
    var Str2 = kpmWidePrepareLowerCaseStr(Text2);
    if (kpmLog_WideSameText)
      kpmLog.logMessage(
        'Text1: <' + Text1 + '>, Text2: <' + Text2 + '>, ' +
        'Str1: <' + Str1 + '>, Str2: <' + Str2 + '>, Str1 == Str2: ' + (Str1 == Str2)
      );
    return Str1 == Str2;
  }
  catch (ErrorMessage)
  {
    if (kpmDebug_Errors)
      kpmLog.logError('kpmWideSameText() Error: ' + ErrorMessage);
    return false;
  }
}
  
function kpmWideSearchText(ASearchingText, AText)
{
  try
  {
    var SearchingText = kpmWidePrepareLowerCaseStr(ASearchingText);
    var Text = kpmWidePrepareLowerCaseStr(AText);
    if (kpmLog_WideSearchText)
      kpmLog.logMessage(
        'ASearchingText: <' + ASearchingText + '>, AText: <' + AText + '>, ' +
        'SearchingText: <' + SearchingText + '>, Text: <' + Text + '>, Text.indexOf(SearchingText): ' + Text.indexOf(SearchingText));
    if (Text.indexOf(SearchingText) >= 0)
      return true;
    else
      return false;
  }
  catch (ErrorMessage)
  {
    if (kpmDebug_Errors)
      kpmLog.logError('kpmWideSearchText() Error: ' + ErrorMessage);
    return false;
  }
}

function kpmTrim(sString)
{
  if (typeof sString != 'string')
    return sString;
  while (sString.substring(0, 1) == ' ')
    sString = sString.substring(1, sString.length);
  while (sString.substring(sString.length-1, sString.length) == ' ')
    sString = sString.substring(0, sString.length-1);
  return sString;
}

function kpmCompareUrl(AUrl1, AUrl2)
{
  try
  {
    AUrl1 = kpmWidePrepareStr(AUrl1);
    AUrl2 = kpmWidePrepareStr(AUrl2);
    // canonicalize Url
    AUrl1 = AUrl1.replace(/%7C/gi, "|");
    AUrl2 = AUrl2.replace(/%7C/gi, "|");
    // compare Url
    if (kpmWideSameText(AUrl1, AUrl2) ||
        kpmWideSearchText(AUrl1, AUrl2) ||
        kpmWideSearchText(AUrl2, AUrl1)
       )
      return true;
    else
      return false;
  }
  catch (ErrorMessage)
  {
    if (kpmDebug_Errors)
      kpmLog.logError('kpmCompareUrl() Error: ' + ErrorMessage);
    return false;
  }
}

// << Strings Script ===========================================================
